Package: MLInterfaces
Title: Uniform interfaces to R machine learning procedures for data in
        Bioconductor containers
Version: 1.92.0
Authors@R: c(
        person("Vincent", "Carey", role=c("cre", "aut"), comment = c(ORCID = "0000-0003-4046-0063"), email="stvjc@channing.harvard.edu"),
        person("Jess", "Mar", role=c("aut")),
	person("Jason", "Vertrees", role=c("ctb")),
	person("Laurent", "Gatto", role=c("ctb")),
        person("Phylis", "Atieno",role = "ctb",
        comment = "Translated vignettes from Sweave to Rmarkdown / HTML."
          ))
Description: This package provides uniform interfaces to machine 
	learning code for data in R and Bioconductor containers.
License: LGPL
Depends: R (>= 3.5), Rcpp, methods, BiocGenerics (>= 0.13.11), Biobase,
        annotate, cluster
Imports: gdata, pls, sfsmisc, MASS, rpart, genefilter, fpc, ggvis,
        shiny, gbm, RColorBrewer, hwriter, threejs (>= 0.2.2), mlbench,
        stats4, tools, grDevices, graphics, stats, magrittr,
        SummarizedExperiment
Suggests: class, e1071, ipred, randomForest, gpls, pamr, nnet, ALL,
        hgu95av2.db, som, hu6800.db, lattice, caret (>= 5.07),
        golubEsets, ada, keggorthology, kernlab, mboost, party, klaR,
        BiocStyle, knitr, testthat, airway
Enhances: parallel
LazyLoad: yes
biocViews: Classification, Clustering
NeedsCompilation: no
Packaged: 2026-04-28 23:19:39 UTC; biocbuild
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/MLInterfaces
git_branch: RELEASE_3_23
git_last_commit: 5f7da61
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
Author: Vincent Carey [cre, aut] (ORCID:
    <https://orcid.org/0000-0003-4046-0063>),
  Jess Mar [aut],
  Jason Vertrees [ctb],
  Laurent Gatto [ctb],
  Phylis Atieno [ctb] (Translated vignettes from Sweave to Rmarkdown /
    HTML.)
Maintainer: Vincent Carey <stvjc@channing.harvard.edu>
