exportPattern("..*")

# from CHECK of 3/25/2020

  importFrom("grDevices", "colorRampPalette")
  importFrom("graphics", "axis", "barplot", "image", "legend", "pairs",
             "par", "points", "text")
  importFrom("methods", "is", "new")
  importFrom("stats", "as.formula", "cutree", "dist", "hclust", "mad",
             "median", "model.frame", "model.matrix", "model.response",
             "na.fail", "prcomp", "predict", "quantile", "terms", "var")
  importFrom("magrittr", "%>%")
  import(SummarizedExperiment)

#import("methods")
#import("tools")
#import("BiocGenerics", except="var")
#import("Biobase")
#import("annotate")
#import("MASS")
#import("rpart")
#import("sfsmisc")
#import("pls")
#import("shiny")
#import("ggvis")
#import("fpc")
#import(threejs)
#import(mlbench)
#
#importFrom(gbm, gbm, predict.gbm)
#
#importFrom("genefilter", "rowttests")
#importFrom("base", "plot")
#importFrom("cluster", "silhouette")
#importFrom("gdata", "drop.levels")
#
#importFrom("RColorBrewer", "brewer.pal")
#
#importFrom("hwriter", "hwrite")
#
#exportClasses( 
#	"varImpStruct",
##	"raboostCont", "daboostCont",
#	"classifierOutput", "learnerSchema", #"nonstandardLearnerSchema",
#	"xvalSpec", "clusteringOutput")
#	
#exportMethods("show", "RObject", "confuMat", "trainInd",
#              "precision", "recall", "sensitivity", "macroF1",
#              "F1", "acc", "tp", "tn", "fp", "fn", "specificity",
#              "MLearn", "planarPlot", "Predict", "getVarImp", 
#              "trainScores","trainPredictions",
#              "testScores", "testPredictions",
#              "predictions", "predScores", "predScore", 
#              "fsHistory", "report", "getDist", "getConverter")
#
#export(randomForestI, knnI, dldaI, nnetI, rpartI, 
#       ldaI, svmI, ldaI.predParms, qdaI, glmI.logistic, 
#       RABI, lvqI, naiveBayesI, baggingI, knn2, dlda2, 
#       rab, lvq, RAB, DAB, plsda2, svm2, ksvm2, 
#       xvalSpec, getVarImp, balKfold.xvspec, 
#       makeLearnerSchema, standardMLIConverter, sldaI, 
#       fs.absT, fs.probT, ksvmI, knn.cvI, knn.cv2, 
#       adaI, fs.topVariance, hclustI, kmeansI, pamI,
#       blackboostI,
#       gbm2, BgbmI, plsdaI,
#       confuTab) 
#
#S3method("predict", "classifierOutput")
#
#exportMethods("plot")
#export(hclustWidget, mlearnWidget)
#
#exportClasses("projectedLearner")
#export(projectLearnerToGrid)
#exportMethods("learnerIn3D", "plotOne")
#
#export(plspinHcube)
#
## recommended
#importFrom("grDevices", "colorRampPalette")
#importFrom("graphics", "axis", "barplot", "legend", "pairs", "par",
#             "points", "text")
#importFrom("stats", "as.formula", "cutree", "dist", "hclust", "median",
#             "model.frame", "model.matrix", "model.response", "na.fail",
#             "prcomp", "predict", "quantile", "terms", "var")
#
